﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Csla;
using Csla.Serialization;
using System.Data;
using System.Data.SqlClient;
using Csla.Data;

namespace Library.Rapport.Samling
{
    /// <summary>
    /// Pristype
    /// </summary>
    [Serializable]
    public class RapportKategori : BusinessBase<RapportKategori>
    {
        #region " Business Properties and Methods "

        private static readonly PropertyInfo<int> RAKRecProperty = RegisterProperty<int>(p => p.RAKRec, "Rec");
        /// <summary>
        /// Unikt rekordnummer for tabellen RapportKategori
        /// </summary>
        public int RAKRec
        {
            get { return GetProperty(RAKRecProperty); }
            set { SetProperty(RAKRecProperty, value); }
        }

        private static readonly PropertyInfo<int> RAKNumberProperty = RegisterProperty<int>(p => p.RAKNumber, "Number");
        /// <summary>
        /// Unikt rekordnummer for tabellen RapportKategori
        /// </summary>
        public int RAKNumber
        {
            get { return GetProperty(RAKNumberProperty); }
            set { SetProperty(RAKNumberProperty, value); }
        }

        private static readonly PropertyInfo<string> RAKNavnProperty = RegisterProperty<string>(p => p.RAKNavn, "Navn");
        /// <summary>
        /// Navn på RapportKategori
        /// </summary>
        public string RAKNavn
        {
            get { return GetProperty(RAKNavnProperty); }
            set { SetProperty(RAKNavnProperty, value); }
        }

        private static readonly PropertyInfo<int> RAKPriorityProperty = RegisterProperty<int>(p => p.RAKPriority, "RAKPriority", 0);
        /// <summary>
        /// Default=0. Høyest nummer kommer på toppen av listen.
        /// </summary>
        public int RAKPriority
        {
            get { return GetProperty(RAKPriorityProperty); }
            set { SetProperty(RAKPriorityProperty, value); }
        }

        private static readonly PropertyInfo<DateTime> DateCreatedProperty = RegisterProperty<DateTime>(p => p.DateCreated, "DateCreated");
        /// <summary>
        /// Dato for opprettelse av post
        /// </summary>
        public DateTime DateCreated
        {
            get { return GetProperty(DateCreatedProperty); }
            set { SetProperty(DateCreatedProperty, value); }
        }

        private static readonly PropertyInfo<DateTime> DateChangedProperty = RegisterProperty<DateTime>(p => p.DateChanged, "DateChanged");
        /// <summary>
        /// Dato for siste endring av post
        /// </summary>
        public DateTime DateChanged
        {
            get { return GetProperty(DateChangedProperty); }
            set { SetProperty(DateChangedProperty, value); }
        }

        private static readonly PropertyInfo<byte[]> LastChangedProperty = RegisterProperty<byte[]>(p => p.LastChanged, "LastChanged");
        /// <summary>
        /// Timestamp for siste endring av post
        /// </summary>
        public byte[] LastChanged
        {
            get { return ReadProperty(LastChangedProperty); }
            internal set { SetProperty(LastChangedProperty, value); }
        }

        #endregion

        #region " Can write property "

        protected override void OnPropertyChanging(Csla.Core.IPropertyInfo propertyInfo)
        {
            base.OnPropertyChanging(propertyInfo);
        }

        #endregion

        #region " Factory Methods "

        public static RapportKategori NewRapportKategori()
        {
            return DataPortal.Create<RapportKategori>();
        }

        /// <summary>
        /// Henter RapportKategori med gitt rekordnummer
        /// </summary>
        /// <param name="RAKRec">Unikt rekordnummer</param>
        /// <returns></returns>
        public static RapportKategori GetRapportKategori(int RAKRec)
        {
            return DataPortal.Fetch<RapportKategori>(RAKRec);
        }

        #endregion

        #region " Data access "

        #region " Data Access - Create "

        [RunLocal]
        protected override void DataPortal_Create()
        {
            using (BypassPropertyChecks)
            {
                ExecuteCreate();
            }
            BusinessRules.CheckRules();
        }

        /// <summary>
        /// Setter standardverdier for klassen
        /// </summary>
        private void ExecuteCreate()
        {
            //RAKPriority = NextNumber();
            base.DataPortal_Create();
        }

        #endregion

        #region " Data Access - Fetch "

        private void DataPortal_Fetch(int RAKRec)
        {
            //using (var ctx = ConnectionManager<SqlConnection>.GetManager("Butikkdata"))
            //{
            //    using (var cm = ctx.Connection.CreateCommand())
            //    {
            //        cm.CommandType = System.Data.CommandType.StoredProcedure;
            //        cm.CommandText = "MGRapportRapportKategori_get";
            //        cm.Parameters.AddWithValue("@RAKRec", RAKRec);
            //        using (var dr = new SafeDataReader(cm.ExecuteReader()))
            //        {
            //            FetchObject(dr);
            //        }
            //    }
            //}
            //BusinessRules.CheckRules();
        }

        private void FetchObject(SafeDataReader dr)
        {
            if (dr.Read())
            {
                using (BypassPropertyChecks)
                {
                    LoadProperty(RAKRecProperty, dr.GetInt32("RAKRec"));
                    LoadProperty(RAKNumberProperty, dr.GetInt32("RAKNumber"));
                    LoadProperty(RAKNavnProperty, dr.GetString("RAKNavn"));
                    LoadProperty(RAKPriorityProperty, dr.GetInt32("RAKPriority"));
                    LoadProperty(DateCreatedProperty, dr.GetDateTime("DateCreated"));
                    LoadProperty(DateChangedProperty, dr.GetDateTime("DateChanged"));
                    LoadProperty(LastChangedProperty, dr.GetValue("LastChanged"));
                }
            }
        }

        #endregion

        #region " Data Access - Insert "


        [Transactional(TransactionalTypes.TransactionScope)]
        protected override void DataPortal_Insert()
        {
            using (var ctx = ConnectionManager<SqlConnection>.GetManager("Butikkdata"))
            {
                using (var cm = ctx.Connection.CreateCommand())
                {
                    cm.CommandType = System.Data.CommandType.StoredProcedure;
                    cm.CommandText = "MGRapportRapportKategori_add";
                    using (BypassPropertyChecks)
                    {
                        cm.Parameters.AddWithValue("@RAKNumber", RAKNumber);
                        cm.Parameters.AddWithValue("@RAKNavn", RAKNavn);
                        cm.Parameters.AddWithValue("@RAKPriority", RAKPriority);

                        cm.Parameters.Add("@DateCreated", SqlDbType.DateTime);
                        cm.Parameters["@DateCreated"].Direction = ParameterDirection.Output;
                        cm.Parameters.Add("@DateChanged", SqlDbType.DateTime);
                        cm.Parameters["@DateChanged"].Direction = ParameterDirection.Output;
                        cm.Parameters.Add("@new_LastChanged", SqlDbType.Timestamp);
                        cm.Parameters["@new_LastChanged"].Direction = ParameterDirection.Output;
                        cm.Parameters.Add("@new_RAKRec", SqlDbType.Int);
                        cm.Parameters["@new_RAKRec"].Direction = ParameterDirection.Output;
                    }

                    cm.ExecuteNonQuery();

                    LoadProperty(RAKRecProperty, cm.Parameters["@new_RAKRec"].Value);
                    LoadProperty(DateCreatedProperty, cm.Parameters["@DateCreated"].Value);
                    LoadProperty(DateChangedProperty, cm.Parameters["@DateChanged"].Value);
                    LoadProperty(LastChangedProperty, cm.Parameters["@new_LastChanged"].Value);
                }
            }

        }


        #endregion

        #region " Data Access - Update "


        [Transactional(TransactionalTypes.TransactionScope)]
        protected override void DataPortal_Update()
        {
            using (var ctx = ConnectionManager<SqlConnection>.GetManager("Butikkdata"))
            {
                using (var cm = ctx.Connection.CreateCommand())
                {
                    cm.CommandType = System.Data.CommandType.StoredProcedure;
                    cm.CommandText = "MGRapportRapportKategori_update";
                    using (BypassPropertyChecks)
                    {
                        cm.Parameters.AddWithValue("@RAKRec", RAKRec);
                        cm.Parameters.AddWithValue("@RAKNumber", RAKNumber);

                        cm.Parameters.AddWithValue("@RAKPriority", RAKPriority);

                        cm.Parameters.AddWithValue("@RAKNavn", RAKNavn);
                        cm.Parameters.AddWithValue("@LastChanged", LastChanged);

                        cm.Parameters.Add("@DateChanged", SqlDbType.DateTime);
                        cm.Parameters["@DateChanged"].Direction = ParameterDirection.Output;
                        cm.Parameters.Add("@new_LastChanged", SqlDbType.Timestamp);
                        cm.Parameters["@new_LastChanged"].Direction = ParameterDirection.Output;
                    }
                    cm.ExecuteNonQuery();

                    LoadProperty(DateChangedProperty, cm.Parameters["@DateChanged"].Value);
                    LoadProperty(LastChangedProperty, cm.Parameters["@new_LastChanged"].Value);
                }
            }

        }


        #endregion

        #region " Data Access - Delete "


        [Transactional(TransactionalTypes.TransactionScope)]
        protected override void DataPortal_DeleteSelf()
        {
            using (BypassPropertyChecks)
            {
                DataPortal_Delete(RAKRec);
            }
        }

        [Transactional(TransactionalTypes.TransactionScope)]
        private void DataPortal_Delete(int RAKRec)
        {
            using (var ctx = ConnectionManager<SqlConnection>.GetManager("Butikkdata"))
            {
                using (var cm = ctx.Connection.CreateCommand())
                {
                    cm.CommandType = System.Data.CommandType.StoredProcedure;
                    cm.CommandText = "MGRapportRapportKategori_delete";
                    using (BypassPropertyChecks)
                    {
                        cm.Parameters.AddWithValue("@RAKRec", RAKRec);
                        cm.Parameters.AddWithValue("@LastChanged", LastChanged);
                        cm.ExecuteNonQuery();
                    }
                }
            }
        }


        #endregion

        #endregion

        #region " dataportal events "

        protected override void DataPortal_OnDataPortalInvoke(DataPortalEventArgs e)
        {
            // implement your behavior here
            base.DataPortal_OnDataPortalInvoke(e);
        }

        protected override void DataPortal_OnDataPortalInvokeComplete(DataPortalEventArgs e)
        {
            // implement your behavior here
            base.DataPortal_OnDataPortalInvokeComplete(e);
        }

        protected override void Child_OnDataPortalException(DataPortalEventArgs e, Exception ex)
        {
            // implement your behavior here
            base.Child_OnDataPortalException(e, ex);
        }

        #endregion

    }

    


}

